import json

import numpy as np

import func_aux as fa

"""
Programa de análisis de sensibilidad dados un número aleatorio de variables
y una ecuación de verificación (métodos de nivel II).

Información de partida:
    1. Parámetros de diseño para la ecuación de verificación.
    2. Propiedades del terreno.
    3. Parámetros de localización, escala y forma, si procede, de los modelos de
       probabilidad que definen las variables.

"""

# Lectura de los datos
info = json.load(open("info_ini.txt", "r"))

# Inicializamos variables y le damos el valor inicial
var, ec_eval = fa.iniciacion_variables(info)
for ind_, var_ in enumerate(info["variables"]):
    var[var_].append(info["pto_critico_ini"][ind_])

dic_aux = dict()

# Iniciamos los valores de comprobación de convergencia de la solución
# Para verificar que la solución ha convergido.
comprobacion = 1e-4
noiter, eps = 0, 10  # Contador del número de iteraciones

while comprobacion < eps:

    # Calculamos mu, sg de la gaussiana que se parece en un entorno del punto
    var = fa.trans_gaussiana(var[var_][noiter], info, var)

    # Aproximación del punto crítico en las nuevas variables, y sus derivadas parciales,
    var = fa.normalizacion(noiter, var, info)
    var = fa.derivada(ec_eval, info, var)

    # Evalúo la ecuación de verificación y las derivadas parciales

    # Creo un diccionario auxiliar para evaluar los valores de las variables
    for ind_, var_ in enumerate(info["variables"]):
        dic_aux[var_] = var[var_][noiter]

    for ind_, var_ in enumerate(info["variables"]):
        var["dh_dyi"][var_] = (
            fa.evalua(
                var["dh_dyi"][var_],
                info["parametros_ec_verificacion"],
                dic_aux,
            )
            * var["desv_tip"][var_]
        )

    # Cálculo del valor de la ecuación de verificación en el punto (yai, ybi)
    var["G"].append(fa.evalua(ec_eval, info["parametros_ec_verificacion"], dic_aux))

    # Cálculo de los coeficientes de la función 'linealizada'
    var, coef = fa.coeficientes(var["G"][noiter], info["variables"], var, noiter)

    epsilon = []
    for i, j in enumerate(info["variables"]):
        epsilon.append(float(var[j][noiter] - var[j][noiter - 1]))
    eps = np.sqrt(np.sum(np.asarray(epsilon) ** 2))

    noiter += 1

# Parte gráfica
fa.panels_graph(var, info)
